//William Barratt
//Copyright 2002 William Barratt
//destegFr.cpp
/* User-friendly steganography decoding program for .pgm files
   Companion to stegFr.cpp */

#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

void handleArgs(int argc, char* argv[], FILE *& infile);
void readThroughHeaders(FILE * infile, int & h, int & w);
void readIntoVector(FILE * infile, int* vector);
void interpretVector(int* vector, char * msg);
void outputMessage(char * msg);

int main(int argc, char* argv[])
{
  FILE * infile;                     //source file
  
  handleArgs(argc, argv, infile);    //get filename
  
  int h, w;                          //image height, width
  
  readThroughHeaders(infile, h, w);  //deal with headers
  
  int * vector = new int[h*w];       //vector for pixel values
    
  readIntoVector(infile, vector);    //read pixel values
  
  fclose(infile);                    //close source file
  
  char * msg = new char[h*w/7];      //string for message
  
  interpretVector(vector, msg);      //decode message
  
  outputMessage(msg);                //print message to screen

  return 0;
}

void handleArgs(int argc, char * argv[], FILE *& infile)
{
  char* file = new char[80];         //temporary string

  if(argc > 2)                       //if user enters 2 things
  {
    cout << "Too many command-line arguments; aborting."
         << endl;
    exit(3);
  }
  else if(argc == 2)                 //if user enters 1 filename
  {
    infile = fopen(argv[1], "r");
  }
  else if(argc == 1)                 //if user enters no name
  {
    cout << "Name of source image file: ";
    cin >> file;
    infile = fopen(file, "r");
  }
}

void readThroughHeaders(FILE * infile, int & h, int & w)
{
  char* str = new char[80];          //string for reading

  fgets(str, 80, infile);            //reads first line
  
  if(str[0] != 'P' || str[1] != '2') //check header
  {
    fclose(infile);
    
    cout << "File header indicates wrong file type" << endl
         << "File must be .pgm with 'P2' in first line" << endl;

    exit(1);
  }

  fgets(str, 80, infile);            //second line (comments)
  while(str[0] == '#')
  {
    fgets(str, 80, infile);          //read through comments
  }

    //retrieval of width and height
  char* width = new char[80];
  char* height = new char[80];

  int spaceLoc = strcspn(str, " ");  //finds space between w & h
  for(int x = 0; x < spaceLoc; x++)
  {
    width[x] = str[x];
  }
  w = atoi(width);                   //convert to int

  height = strstr(str, " ");         //gets height substring
  h = atoi(height);                  //convert to int

  fgets(str, 80, infile);            //line with max value
}

void readIntoVector(FILE *infile, int* vector)
{
    //identical to like-named function of steg.cpp
    //
  char* line = new char[80];         //buffer for reading lines
  char* tmpstr = new char[80];       //temp for manipulation

  int place = 0;                     //vector index
  
  while(!feof(infile))               //while not at file end
  {
    fgets(line, 80, infile);         //get one line
    tmpstr = strtok(line, " ");      //gets pixel value
    
    while(tmpstr != NULL)            //while not at line end
    {
      if(strcspn(tmpstr, "0123456789") != strlen(tmpstr))
      {                              //checks for numbers in substring
        vector[place] = atoi(tmpstr);//places number in vector
        place++;                     //increments index
      }
      tmpstr = strtok(NULL, " ");    //find next value
    }
  }
}

void interpretVector(int * vector, char * msg)
{
  char tmp = 0;                      //buffer char.

  int vectorIndex = 0, strIndex = 0, shiftNum;
  
  while(tmp != char(7))              //while end not reached
  {
    tmp = 0;                         //reset buffer
    
      //runs through, bit-masking etc.
    for(shiftNum = 6; shiftNum >= 0; shiftNum--)
    {
      tmp = tmp ^ ((vector[vectorIndex] & 1) << shiftNum);
      vectorIndex++;                 //increment place in vector
    }
    if(tmp != char(7))
      msg[strIndex] = tmp;           //set output char (if not 7)
    strIndex++;                      //increment place in string
  }
}

void outputMessage(char * msg)
{
  cout << "The message extracted from the file is:" 
       << endl << msg << endl;
}
